@@ -17,8 +17,8 @@
 #' the size of a set we will use cardinality, e.g., the number of actions is
 #' \eqn{|A|}.
 #' 
-#' Note that it is also common in the literature to use for the set of observations 
-#' the letter \eqn{Z}.
+#' Note that the observation model is in the literature
+#' often also denoted by the letter \eqn{Z}.
 #'
 #' **Names used for mathematical symbols in code**
 #'
@@ -52,7 +52,7 @@
 #' * A function with the same arguments are `T_()`, but no default values
 #'   that returns the transition probability.
 #'
-#' **Specification of observation probabilities: \eqn{O(o | s', a)}**
+#' **Specification of observation probabilities: \eqn{O(o | a, s')}**
 #'
 #' The POMDP specifies the probability for each observation \eqn{o} given an
 #' action \eqn{a} and that the system transitioned to the end state
@@ -108,6 +108,14 @@
 #' The default initial belief is a uniform
 #' distribution over all states.
 #'
+#' **Convergence**
+#' 
+#' A infinite-horizon POMDP needs to converge to provide a valid value 
+#' function and policy.
+#' 
+#' A finite-horizon POMDP may also converging to a infinite horizon solution 
+#' if the horizon is long enough.
+#'
 #' **Time-dependent POMDPs**
 #'
 #' Time dependence of transition probabilities, observation probabilities and
@@ -138,7 +146,7 @@
 #' horizon.
 #' @param terminal_values a vector with the terminal values for each state or a
 #' matrix specifying the terminal rewards via a terminal value function (e.g.,
-#' the alpha component produced by solve_POMDP).  A single 0 specifies that all
+#' the alpha component produced by `solve_POMDP()`).  A single 0 specifies that all
 #' terminal values are zero.
 #' @param start Specifies the initial belief state of the agent. A vector with the
 #' probability for each state is supplied. Also the string `'uniform'`
@@ -298,9 +306,9 @@ POMDP <- function(states,
 
 # make sure the definition is complete and everything is in the right order and the right factors
 check_and_fix_MDP <- function(x) {
-  ### TODO: check terminal values
   ### TODO: check function needs to be used!
   ### TODO: keep functions. For now we expand functions into matrices
+  
   check_func <- function(x, func, name) {
     req_formals <- head(names(formals(func)),-1)
     if (!identical(names(formals(x)), req_formals))
@@ -374,7 +382,6 @@ check_and_fix_MDP <- function(x) {
   if (any(x$horizon != floor(x$horizon)))
     stop("horizon needs to be an integer.")
   
-  
   # start
   if (is.numeric(x$start) &&
       length(x$start) == length(x$states)) {
@@ -394,8 +401,6 @@ check_and_fix_MDP <- function(x) {
       )
   }
   
-  ## TODO: check terminal_values
-  
   if ((is.null(x$transition_prob) ||
       (inherits(x, "POMDP") &&
           is.null(x$observation_prob)) ||
@@ -587,6 +592,27 @@ check_and_fix_MDP <- function(x) {
       }
     }
     
+    ### check terminal values (only POMDP for now)
+    if (inherits(x, "POMDP") && !is.null(x$terminal_values)) {
+      if (length(x$terminal_values) != 1L && 
+          length(x$terminal_values) != length(x$states) &&
+          (is.matrix(x$terminal_values) && ncol(x$terminal_values) != length(x$states)))
+        stop("Terminal values are not in the right format.")
+    }
+    
+    ### check solution
+    if (inherits(x, "POMDP") && !is.null(x$solution)) {
+      # alpha
+      if (any(sapply(x$solution$alpha, ncol) != length(x$states)))
+          stop("Alpha vectors do not have the right dimension.")
+      
+      # pg
+      x$solution$pg <- lapply(x$solution$pg, FUN = function(y) {
+        y$action <- factor(y$action, levels = x$actions)
+        y
+      })
+    }
+    
     if (inherits(x, "POMDP") &&
         !is.null(x$observation_prob) &&
         !.is_timedependent_field(x, "observation_prob")) {
@@ -705,10 +731,14 @@ print.POMDP <- function(x, ...) {
     length(x$observations)
   ))
   
+  writeLines(paste0(
+    "  Start: ", shorten(paste(x$start, collapse = ", "), n = -10L)
+  ))
+  
   if (!is.null(x$normalized) && x$normalized)
-    writeLines(c("  Normalized: TRUE", ""))
+    writeLines(c("  Normalized: TRUE"))
   else
-    writeLines(c("  Normalized: FALSE", ""))
+    writeLines(c("  Normalized: FALSE"))
   
   if (is_solved_POMDP(x))
     writeLines(c(
