@@ -26,7 +26,8 @@
 #' @param epsilon the probability of random actions  for using an epsilon-greedy policy.
 #'  Default for solved models is 0 and for unsolved model 1.
 #' @param engine `'cpp'` or `'r'` to perform simulation using a faster C++
-#'  or a native R implementation which supports sparse matrices.
+#'  or a native R implementation.
+#' @param delta_horizon precision used to determine the horizon for infinite-horizon problems.
 #' @param verbose report used parameters.
 #' @param ... further arguments are ignored.
 #' @return A list with elements:
@@ -47,25 +48,33 @@
 #' # solve the POMDP for 5 epochs and no discounting
 #' sol <- solve_MDP(Maze, discount = 1)
 #' sol
-#' policy(sol)
+#' 
 #' # U in the policy is and estimate of the utility of being in a state when using the optimal policy.
+#' policy(sol)
+#' matrix(policy(sol)[[1]]$action, nrow = 3, dimnames = list(1:3, 1:4))[3:1, ]
 #'
-#' ## Example 1: simulate 10 trajectories, only the final belief state is returned
+#' ## Example 1: simulate 10 trajectories following the policy, only the final belief state is returned
 #' sim <- simulate_MDP(sol, n = 100, horizon = 10, verbose = TRUE)
 #' sim
-#'
-#' # Calculate proportion of actions used
+#' 
+#' # Note that all simulations start at s_1 and that the simulated avg. reward 
+#' # is therefore an estimate to the U value for the start state s_1.
+#' policy(sol)[[1]][1,] 
+#' 
+#' # Calculate proportion of actions taken in the simulation
 #' round_stochastic(sim$action_cnt / sum(sim$action_cnt), 2)
 #'
 #' # reward distribution
 #' hist(sim$reward)
 #'
-#' ## Example 2: simulate starting always in state s_1 and return all visited states
-#' sim <- simulate_MDP(sol, n = 100, start = "s_1", horizon = 10, return_states = TRUE)
+#' ## Example 2: simulate starting following a uniform distribution over all
+#' #             states and return all visited states
+#' sim <- simulate_MDP(sol, n = 100, start = "uniform", horizon = 10, return_states = TRUE)
 #' sim$avg_reward
 #'
 #' # how often was each state visited?
 #' table(sim$states)
+#' matrix(table(sim$states),nrow = 3, dimnames = list(1:3, 1:4))[3:1, ]
 #' @export
 simulate_MDP <-
   function(model,
@@ -74,25 +83,29 @@ simulate_MDP <-
     horizon = NULL,
     return_states = FALSE,
     epsilon = NULL,
+    delta_horizon = 1e-3,
     engine = "cpp",
     verbose = FALSE,
     ...) {
     engine <- match.arg(tolower(engine), c("cpp", "r"))
     
-    if (engine == "r")
-      sparse <- NULL   ### use the version in the model
-    else
-      sparse <- FALSE
-    
     start <- .translate_belief(start, model = model)
     solved <- is_solved_MDP(model)
     
+    n <- as.integer(n)
+    
     if (is.null(horizon))
       horizon <- model$horizon
-    if (is.null(horizon))
-      stop("The horizon (number of epochs) has to be specified!")
-    if (is.infinite(horizon))
-      stop("Simulation needs a finite simulation horizon.")
+    if (is.null(horizon) || is.infinite(horizon)) {
+      if (is.null(model$discount) || !(model$discount < 1)) 
+        stop("Simulation needs a finite simulation horizon.")
+      
+      # find a horizon that approximates the reward using 
+      # discount^horizon * max_abs_R <= 0.001
+      max_abs_R <-  max(abs(reward_matrix(model, sparse = TRUE)$value))
+      horizon <- ceiling(log(delta_horizon/max_abs_R)/log(model$discount))
+    }
+    horizon <- as.integer(horizon)
     
     if (is.null(epsilon)) {
       if (!solved)
@@ -109,6 +122,7 @@ simulate_MDP <-
       disc <- 1
     
     if (engine == "cpp") {
+      ### TODO: Make sparse
       model <- normalize_MDP(model, sparse = FALSE)
       
       if (foreach::getDoParWorkers() == 1 || n * horizon < 100000)
@@ -167,8 +181,8 @@ simulate_MDP <-
     n_states <- length(states)
     actions <- as.character(model$actions)
     
-    trans_m <- transition_matrix(model, sparse = sparse)
-    #rew_m <- reward_matrix(model, sparse = sparse)
+    trans_m <- transition_matrix(model, sparse = NULL)
+    #rew_m <- reward_matrix(model, sparse = NULL)
     
     # for easier access
     pol <-
